@@ -1,9 +1,6 @@
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 
 const Clutter = imports.gi.Clutter;
-const Pango = imports.gi.Pango;
-const GLib = imports.gi.GLib;
-const Gio = imports.gi.Gio;
 const Gtk = imports.gi.Gtk;
 const Shell = imports.gi.Shell;
 const Lang = imports.lang;
@@ -15,16 +12,17 @@ const _ = Gettext.gettext;
 
 const AppFavorites = imports.ui.appFavorites;
 const DND = imports.ui.dnd;
-const GenericDisplay = imports.ui.genericDisplay;
+const IconGrid = imports.ui.iconGrid;
 const Main = imports.ui.main;
 const Overview = imports.ui.overview;
 const PopupMenu = imports.ui.popupMenu;
 const Search = imports.ui.search;
 const Tweener = imports.ui.tweener;
 const Workspace = imports.ui.workspace;
+const Params = imports.misc.params;
 
-const APPICON_SIZE = 48;
-const WELL_MAX_COLUMNS = 8;
+const WELL_MAX_COLUMNS = 16;
+const WELL_MAX_SEARCH_ROWS = 1;
 const MENU_POPUP_TIMEOUT = 600;
 
 function AlphabeticalView() {
@@ -34,7 +32,7 @@ function AlphabeticalView() {
 AlphabeticalView.prototype = {
     _init: function() {
         this.actor = new St.BoxLayout({ vertical: true });
-        this._grid = new WellGrid(true);
+        this._grid = new IconGrid.IconGrid();
         this._appSystem = Shell.AppSystem.get_default();
         this.actor.add(this._grid.actor, { y_align: St.Align.START, expand: true });
     },
@@ -224,105 +222,36 @@ AppSearchResultDisplay.prototype = {
 
     _init: function (provider) {
         Search.SearchResultDisplay.prototype._init.call(this, provider);
-        this._spacing = 0;
+        this._grid = new IconGrid.IconGrid({ rowLimit: WELL_MAX_SEARCH_ROWS });
         this.actor = new St.Bin({ name: 'dashAppSearchResults',
                                   x_align: St.Align.START });
-        this.actor.connect('style-changed', Lang.bind(this, this._onStyleChanged));
-        let container = new Shell.GenericContainer();
-        this._container = container;
-        this.actor.set_child(container);
-        container.connect('get-preferred-width', Lang.bind(this, this._getPreferredWidth));
-        container.connect('get-preferred-height', Lang.bind(this, this._getPreferredHeight));
-        container.connect('allocate', Lang.bind(this, this._allocate));
-    },
-
-    _getPreferredWidth: function (actor, forHeight, alloc) {
-        let children = actor.get_children();
-
-        for (let i = 0; i < children.length; i++) {
-            let [minSize, natSize] = children[i].get_preferred_width(forHeight);
-            alloc.natural_size += natSize;
-        }
-    },
-
-    _getPreferredHeight: function (actor, forWidth, alloc) {
-        let children = actor.get_children();
-
-        for (let i = 0; i < children.length; i++) {
-            let [minSize, natSize] = children[i].get_preferred_height(forWidth);
-            if (minSize > alloc.min_size)
-                alloc.min_size = minSize;
-            if (natSize > alloc.natural_size)
-                alloc.natural_size = natSize;
-        }
-    },
-
-    _allocate: function (actor, box, flags) {
-        let availWidth = box.x2 - box.x1;
-        let availHeight = box.y2 - box.y1;
-
-        let children = actor.get_children();
-
-        let x = 0;
-        let i;
-        for (i = 0; i < children.length; i++) {
-            let child = children[i];
-            let childBox = new Clutter.ActorBox();
-
-            let [minWidth, minHeight, natWidth, natHeight] = child.get_preferred_size();
-
-            if (x + natWidth > availWidth) {
-                actor.set_skip_paint(child, true);
-                continue;
-            }
-
-            let yPadding = Math.max(0, availHeight - natHeight);
-
-            childBox.x1 = x;
-            childBox.x2 = childBox.x1 + natWidth;
-            childBox.y1 = Math.floor(yPadding / 2);
-            childBox.y2 = availHeight - childBox.y1;
-
-            x = childBox.x2 + this._spacing;
-
-            child.allocate(childBox, flags);
-            actor.set_skip_paint(child, false);
-        }
-    },
-
-    _onStyleChanged: function () {
-        let themeNode = this.actor.get_theme_node();
-        let [success, len] = themeNode.get_length('spacing', false);
-        if (success)
-            this._spacing = len;
-        this._container.queue_relayout();
+        this.actor.set_child(this._grid.actor);
     },
 
     renderResults: function(results, terms) {
         let appSys = Shell.AppSystem.get_default();
-        for (let i = 0; i < results.length && i < WELL_MAX_COLUMNS; i++) {
+        let maxItems = WELL_MAX_SEARCH_ROWS * WELL_MAX_COLUMNS;
+        for (let i = 0; i < results.length && i < maxItems; i++) {
             let result = results[i];
             let app = appSys.get_app(result);
             let display = new AppWellIcon(app);
-            this._container.add_actor(display.actor);
+            this._grid.addItem(display.actor);
         }
     },
 
     clear: function () {
-        this._container.get_children().forEach(function (actor) { actor.destroy(); });
+        this._grid.removeAll();
         this.selectionIndex = -1;
     },
 
     getVisibleResultCount: function() {
-        let nChildren = this._container.get_children().length;
-        return nChildren - this._container.get_n_skip_paint();
+        return this._grid.visibleItemsCount();
     },
 
     selectIndex: function (index) {
         let nVisible = this.getVisibleResultCount();
-        let children = this._container.get_children();
         if (this.selectionIndex >= 0) {
-            let prevActor = children[this.selectionIndex];
+            let prevActor = this._grid.getItemAtIndex(this.selectionIndex);
             prevActor._delegate.setSelected(false);
         }
         this.selectionIndex = -1;
@@ -330,7 +259,7 @@ AppSearchResultDisplay.prototype = {
             return false;
         else if (index < 0)
             return false;
-        let targetActor = children[index];
+        let targetActor = this._grid.getItemAtIndex(index);
         targetActor._delegate.setSelected(true);
         this.selectionIndex = index;
         return true;
@@ -339,8 +268,7 @@ AppSearchResultDisplay.prototype = {
     activateSelected: function() {
         if (this.selectionIndex < 0)
             return;
-        let children = this._container.get_children();
-        let targetActor = children[this.selectionIndex];
+        let targetActor = this._grid.getItemAtIndex(this.selectionIndex);
         this.provider.activateResult(targetActor._delegate.app.get_id());
     }
 };
@@ -440,23 +368,18 @@ function AppIcon(app) {
 }
 
 AppIcon.prototype = {
+    __proto__:  IconGrid.BaseIcon.prototype,
+
     _init : function(app) {
         this.app = app;
 
-        this.actor = new St.Bin({ style_class: 'app-icon',
-                                  x_fill: true,
-                                  y_fill: true });
-        this.actor._delegate = this;
+        let label = this.app.get_name();
 
-        let box = new St.BoxLayout({ vertical: true });
-        this.actor.set_child(box);
+        IconGrid.BaseIcon.prototype._init.call(this, label);
+    },
 
-        this.icon = this.app.create_icon_texture(APPICON_SIZE);
-
-        box.add(this.icon, { expand: true, x_fill: false, y_fill: false });
-
-        this._name = new St.Label({ text: this.app.get_name() });
-        box.add_actor(this._name);
+    createIcon: function(iconSize) {
+        return this.app.create_icon_texture(iconSize);
     }
 };
 
@@ -659,7 +582,7 @@ AppWellIcon.prototype = {
     },
 
     getDragActor: function() {
-        return this.app.create_icon_texture(APPICON_SIZE);
+        return this.app.create_icon_texture(this._icon.iconSize);
     },
 
     // Returns the original actor that should align with the actor
@@ -835,136 +758,6 @@ AppIconMenu.prototype = {
 };
 Signals.addSignalMethods(AppIconMenu.prototype);
 
-function WellGrid() {
-    this._init();
-}
-
-WellGrid.prototype = {
-    _init: function() {
-        this.actor = new St.BoxLayout({ name: 'dashAppWell', vertical: true });
-        // Pulled from CSS, but hardcode some defaults here
-        this._spacing = 0;
-        this._item_size = 48;
-        this._grid = new Shell.GenericContainer();
-        this.actor.add(this._grid, { expand: true, y_align: St.Align.START });
-        this.actor.connect('style-changed', Lang.bind(this, this._onStyleChanged));
-
-        this._grid.connect('get-preferred-width', Lang.bind(this, this._getPreferredWidth));
-        this._grid.connect('get-preferred-height', Lang.bind(this, this._getPreferredHeight));
-        this._grid.connect('allocate', Lang.bind(this, this._allocate));
-    },
-
-    _getPreferredWidth: function (grid, forHeight, alloc) {
-        let children = this._grid.get_children();
-        let nColumns = children.length;
-        let totalSpacing = Math.max(0, nColumns - 1) * this._spacing;
-        // Kind of a lie, but not really an issue right now.  If
-        // we wanted to support some sort of hidden/overflow that would
-        // need higher level design
-        alloc.min_size = this._item_size;
-        alloc.natural_size = nColumns * this._item_size + totalSpacing;
-    },
-
-    _getPreferredHeight: function (grid, forWidth, alloc) {
-        let children = this._grid.get_children();
-        let [nColumns, usedWidth] = this._computeLayout(forWidth);
-        let nRows;
-        if (nColumns > 0)
-            nRows = Math.ceil(children.length / nColumns);
-        else
-            nRows = 0;
-        let totalSpacing = Math.max(0, nRows - 1) * this._spacing;
-        let height = nRows * this._item_size + totalSpacing;
-        alloc.min_size = height;
-        alloc.natural_size = height;
-    },
-
-    _allocate: function (grid, box, flags) {
-        let children = this._grid.get_children();
-        let availWidth = box.x2 - box.x1;
-        let availHeight = box.y2 - box.y1;
-
-        let [nColumns, usedWidth] = this._computeLayout(availWidth);
-
-        let overallPaddingX = Math.floor((availWidth - usedWidth) / 2);
-
-        let x = box.x1 + overallPaddingX;
-        let y = box.y1;
-        let columnIndex = 0;
-        for (let i = 0; i < children.length; i++) {
-            let [childMinWidth, childMinHeight, childNaturalWidth, childNaturalHeight]
-                = children[i].get_preferred_size();
-
-            /* Center the item in its allocation horizontally */
-            let width = Math.min(this._item_size, childNaturalWidth);
-            let childXSpacing = Math.max(0, width - childNaturalWidth) / 2;
-            let height = Math.min(this._item_size, childNaturalHeight);
-            let childYSpacing = Math.max(0, height - childNaturalHeight) / 2;
-
-            let childBox = new Clutter.ActorBox();
-            if (St.Widget.get_default_direction() == St.TextDirection.RTL) {
-                let _x = box.x2 - (x + width);
-                childBox.x1 = Math.floor(_x - childXSpacing);
-            } else {
-                childBox.x1 = Math.floor(x + childXSpacing);
-            }
-            childBox.y1 = Math.floor(y + childYSpacing);
-            childBox.x2 = childBox.x1 + width;
-            childBox.y2 = childBox.y1 + height;
-            children[i].allocate(childBox, flags);
-
-            columnIndex++;
-            if (columnIndex == nColumns) {
-                columnIndex = 0;
-            }
-
-            if (columnIndex == 0) {
-                y += this._item_size + this._spacing;
-                x = box.x1 + overallPaddingX;
-            } else {
-                x += this._item_size + this._spacing;
-            }
-        }
-    },
-
-    _computeLayout: function (forWidth) {
-        let children = this._grid.get_children();
-        let nColumns = 0;
-        let usedWidth = 0;
-        while (nColumns < WELL_MAX_COLUMNS &&
-                (usedWidth + this._item_size <= forWidth)) {
-            usedWidth += this._item_size + this._spacing;
-            nColumns += 1;
-        }
-
-        if (nColumns > 0)
-            usedWidth -= this._spacing;
-
-        return [nColumns, usedWidth];
-    },
-
-    _onStyleChanged: function() {
-        let themeNode = this.actor.get_theme_node();
-        let [success, len] = themeNode.get_length('spacing', false);
-        if (success)
-            this._spacing = len;
-        [success, len] = themeNode.get_length('-shell-grid-item-size', false);
-        if (success)
-            this._item_size = len;
-        this._grid.queue_relayout();
-    },
-
-    removeAll: function () {
-        this._grid.get_children().forEach(Lang.bind(this, function (child) {
-            child.destroy();
-        }));
-    },
-
-    addItem: function(actor) {
-        this._grid.add_actor(actor);
-    }
-};
-
 function AppWell() {
     this._init();
 }
@@ -977,7 +770,7 @@ AppWell.prototype = {
 
         this._favorites = [];
 
-        this._grid = new WellGrid();
+        this._grid = new IconGrid.IconGrid();
         this.actor = this._grid.actor;
         this.actor._delegate = this;
 
@@ -1039,6 +832,29 @@ AppWell.prototype = {
         }
     },
 
+    handleDragOver : function(source, actor, x, y, time) {
+        let app = null;
+        if (source instanceof AppWellIcon)
+            app = this._appSystem.get_app(source.getId());
+        else if (source instanceof Workspace.WindowClone)
+            app = this._tracker.get_window_app(source.metaWindow);
+
+        // Don't allow favoriting of transient apps
+        if (app == null || app.is_transient())
+            return DND.DragMotionResult.NO_DROP;
+
+        let id = app.get_id();
+
+        let favorites = AppFavorites.getAppFavorites().getFavoriteMap();
+
+        let srcIsFavorite = (id in favorites);
+
+        if (srcIsFavorite)
+            return DND.DragMotionResult.NO_DROP;
+
+        return DND.DragMotionResult.COPY_DROP;
+    },
+
     // Draggable target interface
     acceptDrop : function(source, actor, x, y, time) {
         let app = null;
