@@ -2,9 +2,9 @@
 
 imports.gi.versions.Clutter = '1.0';
 imports.gi.versions.Gio = '2.0';
-imports.gi.versions.Gdk = '2.0';
+imports.gi.versions.Gdk = '3.0';
 imports.gi.versions.GdkPixbuf = '2.0';
-imports.gi.versions.Gtk = '2.0';
+imports.gi.versions.Gtk = '3.0';
 
 const Clutter = imports.gi.Clutter;
 const DBus = imports.dbus;
@@ -16,7 +16,6 @@ const Lang = imports.lang;
 const Mainloop = imports.mainloop;
 const Meta = imports.gi.Meta;
 const Shell = imports.gi.Shell;
-const Signals = imports.signals;
 const St = imports.gi.St;
 
 const Chrome = imports.ui.chrome;
@@ -35,6 +34,7 @@ const ShellDBus = imports.ui.shellDBus;
 const TelepathyClient = imports.ui.telepathyClient;
 const WindowManager = imports.ui.windowManager;
 const Magnifier = imports.ui.magnifier;
+const StatusIconDispatcher = imports.ui.statusIconDispatcher;
 
 const DEFAULT_BACKGROUND_COLOR = new Clutter.Color();
 DEFAULT_BACKGROUND_COLOR.from_pixel(0x2266bbff);
@@ -56,6 +56,7 @@ let modalCount = 0;
 let modalActorFocusStack = [];
 let uiGroup = null;
 let magnifier = null;
+let statusIconDispatcher = null;
 let _errorLogStack = [];
 let _startDate;
 
@@ -125,6 +126,8 @@ function start() {
     placesManager = new PlaceDisplay.PlacesManager();
     overview = new Overview.Overview();
     chrome = new Chrome.Chrome();
+    magnifier = new Magnifier.Magnifier();
+    statusIconDispatcher = new StatusIconDispatcher.StatusIconDispatcher();
     panel = new Panel.Panel();
     wm = new WindowManager.WindowManager();
     messageTray = new MessageTray.MessageTray();
@@ -174,9 +177,6 @@ function start() {
 
     global.stage.connect('captured-event', _globalKeyPressHandler);
 
-    // Install magnifier.
-    magnifier = new Magnifier.Magnifier();
-
     // Perform initial relayout here
     _relayout();
 
