@@ -1,10 +1,8 @@
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 
-const Clutter = imports.gi.Clutter;
 const DBus = imports.dbus;
 const GLib = imports.gi.GLib;
 const Lang = imports.lang;
-const Shell = imports.gi.Shell;
 const Signals = imports.signals;
 const St = imports.gi.St;
 const Gettext = imports.gettext.domain('gnome-shell');
@@ -213,7 +211,7 @@ ContactManager.prototype = {
             throw new Error('Could not parse connection path ' + connPath);
 
         info.cacheDir = this._cacheDir + '/' + match[1];
-        GLib.mkdir_with_parents(info.cacheDir, 0700);
+        GLib.mkdir_with_parents(info.cacheDir, 0x1c0); // 0x1c0 = octal 0700
 
         // info.names[handle] is @handle's real name
         // info.tokens[handle] is the token for @handle's avatar
@@ -349,7 +347,7 @@ ContactManager.prototype = {
             let uri = GLib.filename_to_uri(file, null);
             iconBox.child = textureCache.load_uri_async(uri, iconBox._size, iconBox._size);
         } else {
-            iconBox.child = textureCache.load_icon_name('stock_person', iconBox._size);
+            iconBox.child = textureCache.load_icon_name('stock_person', St.IconType.FULLCOLOR, iconBox._size);
         }
     },
 
@@ -447,7 +445,7 @@ Source.prototype = {
     __proto__:  MessageTray.Source.prototype,
 
     _init: function(accountPath, connPath, channelPath, targetHandle, targetHandleType, targetId) {
-        MessageTray.Source.prototype._init.call(this, targetId, targetId);
+        MessageTray.Source.prototype._init.call(this, targetId);
 
         this._accountPath = accountPath;
 
@@ -477,10 +475,13 @@ Source.prototype = {
         this._receivedId = this._channelText.connect('Received', Lang.bind(this, this._messageReceived));
 
         this._channelText.ListPendingMessagesRemote(false, Lang.bind(this, this._gotPendingMessages));
+
+        this._setSummaryIcon(this.createNotificationIcon());
     },
 
-    createIcon: function(size) {
-        return contactManager.createAvatar(this._conn, this._targetHandle, size);
+    createNotificationIcon: function() {
+        return contactManager.createAvatar(this._conn, this._targetHandle,
+                                           this.ICON_SIZE);
     },
 
     clicked: function() {
@@ -524,7 +525,7 @@ Source.prototype = {
             Main.messageTray.add(this);
 
         if (!this._notification)
-            this._notification = new Notification(this._targetId, this);
+            this._notification = new Notification(this);
     },
 
     _messageReceived: function(channel, id, timestamp, sender,
@@ -570,19 +571,17 @@ Source.prototype = {
     }
 };
 
-function Notification(id, source) {
-    this._init(id, source);
+function Notification(source) {
+    this._init(source);
 }
 
 Notification.prototype = {
     __proto__:  MessageTray.Notification.prototype,
 
-    _init: function(id, source) {
-        MessageTray.Notification.prototype._init.call(this, id, source, source.title);
-        this.actor.connect('button-press-event', Lang.bind(this, this._onButtonPress));
+    _init: function(source) {
+        MessageTray.Notification.prototype._init.call(this, source, source.title, null, { customContent: true });
 
         this._responseEntry = new St.Entry({ style_class: 'chat-response' });
-        this._responseEntry.clutter_text.connect('key-focus-in', Lang.bind(this, this._onEntryFocused));
         this._responseEntry.clutter_text.connect('activate', Lang.bind(this, this._onEntryActivated));
         this.setActionArea(this._responseEntry);
 
@@ -591,9 +590,9 @@ Notification.prototype = {
 
     appendMessage: function(text, asTitle) {
         if (asTitle)
-            this.update(text);
+            this.update(text, null, { customContent: true });
         else
-            this.update(this.source.title, text);
+            this.update(this.source.title, text, { customContent: true });
         this._append(text, 'chat-received');
     },
 
@@ -623,60 +622,20 @@ Notification.prototype = {
         }
     },
 
-    _onButtonPress: function(notification, event) {
-        if (!this._active)
-            return false;
-
-        let source = event.get_source ();
-        if (source && notification.contains(source))
-            return false;
-
-        // @source is outside @notification, which has to mean that
-        // we have a pointer grab, and the user clicked outside the
-        // notification, so we should deactivate.
-        this._deactivate();
-        return true;
-    },
-
-    _onEntryFocused: function() {
-        if (this._active)
-            return;
-
-        if (!Main.pushModal(this.actor))
-            return;
-        Clutter.grab_pointer(this.actor);
-
-        this._active = true;
-        Main.messageTray.lock();
+    grabFocus: function(lockTray) {
+        // Need to call the base class function first so that
+        // it saves where the key focus was before.
+        MessageTray.Notification.prototype.grabFocus.call(this, lockTray);
+        global.stage.set_key_focus(this._responseEntry.clutter_text);
     },
 
     _onEntryActivated: function() {
         let text = this._responseEntry.get_text();
-        if (text == '') {
-            this._deactivate();
+        if (text == '')
             return;
-        }
 
         this._responseEntry.set_text('');
         this._append(text, 'chat-sent');
         this.source.respond(text);
-    },
-
-    _deactivate: function() {
-        if (this._active) {
-            Clutter.ungrab_pointer(this.actor);
-            Main.popModal(this.actor);
-            global.stage.set_key_focus(null);
-
-            // We have to do this after calling popModal(), because
-            // that will return the keyboard focus to
-            // this._responseEntry (because that's where it was when
-            // pushModal() was called), which will cause
-            // _onEntryFocused() to be called again, but we don't want
-            // it to do anything.
-            this._active = false;
-
-            Main.messageTray.unlock();
-        }
     }
 };
