@@ -1,9 +1,7 @@
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 
-const Clutter = imports.gi.Clutter;
 const Lang = imports.lang;
 const Shell = imports.gi.Shell;
-const Meta = imports.gi.Meta;
 const Gettext = imports.gettext.domain('gnome-shell');
 const _ = Gettext.gettext;
 
@@ -16,10 +14,12 @@ function WindowAttentionHandler() {
 
 WindowAttentionHandler.prototype = {
     _init : function() {
+        this._startupIds = {};
+        this._sources = {};
+
         let display = global.screen.get_display();
         display.connect('window-demands-attention', Lang.bind(this, this._onWindowDemandsAttention));
         let tracker = Shell.WindowTracker.get_default();
-        this._startupIds = {};
         tracker.connect('startup-sequence-changed', Lang.bind(this, this._onStartupSequenceChanged));
     },
 
@@ -31,10 +31,6 @@ WindowAttentionHandler.prototype = {
         }
     },
 
-    _sourceId : function(appId) {
-        return 'attention-' + appId;
-    },
-
     _getTitle : function(app, window) {
         if (this._startupIds[window.get_startup_id()])
             return app.get_name();
@@ -63,19 +59,22 @@ WindowAttentionHandler.prototype = {
 
         let tracker = Shell.WindowTracker.get_default();
         let app = tracker.get_window_app(window);
+        let appId = app.get_id();
 
-        let source = Main.messageTray.getSource(this._sourceId(app.get_id()));
+        let source = this._sources[appId];
         if (source == null) {
-            source = new Source(this._sourceId(app.get_id()), app, window);
+            source = new Source(app, window);
+            this._sources[appId] = source;
             Main.messageTray.add(source);
             source.connect('clicked', Lang.bind(this, function() { source.destroy(); }));
+            source.connect('destroy', Lang.bind(this, function() { delete this._sources[appId]; }));
         }
 
-        let notification = new MessageTray.Notification(window.get_startup_id(), source, this._getTitle(app, window), this._getBanner(app, window), true);
+        let notification = new MessageTray.Notification(source, this._getTitle(app, window), this._getBanner(app, window));
         source.notify(notification);
 
         window.connect('notify::title', Lang.bind(this, function(win) {
-                                                            notification.update(this._getTitle(app, win), this._getBanner(app, win), false);
+                                                            notification.update(this._getTitle(app, win), this._getBanner(app, win));
                                                         }));
         window.connect('notify::demands-attention', Lang.bind(this, function() { source.destroy(); }));
         window.connect('focus', Lang.bind(this, function() { source.destroy(); }));
@@ -84,21 +83,22 @@ WindowAttentionHandler.prototype = {
     }
 };
 
-function Source(sourceId, app, window) {
-    this._init(sourceId, app, window);
+function Source(app, window) {
+    this._init(app, window);
 }
 
 Source.prototype = {
     __proto__ : MessageTray.Source.prototype,
 
-    _init: function(sourceId, app, window) {
-        MessageTray.Source.prototype._init.call(this, sourceId, app.get_name());
+    _init: function(app, window) {
+        MessageTray.Source.prototype._init.call(this, app.get_name());
         this._window = window;
         this._app = app;
+        this._setSummaryIcon(this.createNotificationIcon());
     },
 
-    createIcon : function(size) {
-        return this._app.create_icon_texture(size);
+    createNotificationIcon : function() {
+        return this._app.create_icon_texture(this.ICON_SIZE);
     },
 
     clicked : function() {
