@@ -51,7 +51,7 @@
 
 #include "shell-global-private.h"
 #include "shell-perf-log.h"
-#include "shell-wm.h"
+#include "shell-wm-private.h"
 #include "st.h"
 
 static void gnome_shell_plugin_dispose     (GObject *object);
@@ -154,8 +154,12 @@ static void
 gnome_shell_plugin_init (GnomeShellPlugin *shell_plugin)
 {
   g_setenv ("XDG_MENU_PREFIX", "gs-", TRUE);
+  meta_prefs_override_preference_location ("/apps/mutter/general/attach_modal_dialogs",
+                                           "/desktop/gnome/shell/windows/attach_modal_dialogs");
   meta_prefs_override_preference_location ("/apps/metacity/general/button_layout",
                                            "/desktop/gnome/shell/windows/button_layout");
+  meta_prefs_override_preference_location ("/apps/metacity/general/side_by_side_tiling",
+                                           "/desktop/gnome/shell/windows/side_by_side_tiling");
 }
 
 static void
@@ -503,6 +507,21 @@ gnome_shell_plugin_xevent_filter (MutterPlugin *plugin,
     }
 #endif
 
+  /* When the pointer leaves the stage to enter a child of the stage
+   * (like a notification icon), we don't want to produce Clutter leave
+   * events. But Clutter treats all leave events identically, so we
+   * need hide the detail = NotifyInferior events from it.
+   *
+   * Since Clutter doesn't see any event at all, this does mean that
+   * it won't produce an enter event on a Clutter actor that surrounds
+   * the child (unless it gets a MotionNotify before the Enter event).
+   * Other weirdness is likely also possible.
+   */
+  if ((xev->xany.type == EnterNotify || xev->xany.type == LeaveNotify)
+      && xev->xcrossing.detail == NotifyInferior
+      && xev->xcrossing.window == clutter_x11_get_stage_window (CLUTTER_STAGE (clutter_stage_get_default ())))
+    return TRUE;
+
   return clutter_x11_handle_event (xev) != CLUTTER_X11_FILTER_CONTINUE;
 }
 
