@@ -3,7 +3,9 @@
 #include "config.h"
 
 #include "shell-global-private.h"
+#include "shell-enum-types.h"
 #include "shell-perf-log.h"
+#include "shell-window-tracker.h"
 #include "shell-wm.h"
 
 #include "display.h"
@@ -25,6 +27,7 @@
 #ifdef HAVE_SYS_RESOURCE_H
 #include <sys/resource.h>
 #endif
+#include "shell-jsapi-compat-private.h"
 
 #define SHELL_DBUS_SERVICE "org.gnome.Shell"
 #define MAGNIFIER_DBUS_SERVICE "org.gnome.Magnifier"
@@ -57,6 +60,7 @@ struct _ShellGlobal {
 
   /* Displays the root window; see shell_global_create_root_pixmap_actor() */
   ClutterActor *root_pixmap;
+  GdkWindow    *stage_window;
 
   gint last_change_screen_width, last_change_screen_height;
 
@@ -74,6 +78,7 @@ enum {
   PROP_SCREEN_WIDTH,
   PROP_SCREEN_HEIGHT,
   PROP_STAGE,
+  PROP_STAGE_INPUT_MODE,
   PROP_WINDOW_GROUP,
   PROP_WINDOW_MANAGER,
   PROP_SETTINGS,
@@ -90,8 +95,14 @@ shell_global_set_property(GObject         *object,
                           const GValue    *value,
                           GParamSpec      *pspec)
 {
+  ShellGlobal *global = SHELL_GLOBAL (object);
+
   switch (prop_id)
     {
+    case PROP_STAGE_INPUT_MODE:
+      shell_global_set_stage_input_mode (global, g_value_get_enum (value));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -136,6 +147,9 @@ shell_global_get_property(GObject         *object,
     case PROP_STAGE:
       g_value_set_object (value, mutter_plugin_get_stage (global->plugin));
       break;
+    case PROP_STAGE_INPUT_MODE:
+      g_value_set_enum (value, global->input_mode);
+      break;
     case PROP_WINDOW_GROUP:
       g_value_set_object (value, mutter_plugin_get_window_group (global->plugin));
       break;
@@ -194,6 +208,7 @@ shell_global_init (ShellGlobal *global)
   global->gtk_grab_active = FALSE;
 
   global->root_pixmap = NULL;
+  global->stage_window = NULL;
 
   global->input_mode = SHELL_STAGE_INPUT_MODE_NORMAL;
 
@@ -254,6 +269,14 @@ shell_global_class_init (ShellGlobalClass *klass)
                                                         "Stage holding the desktop scene graph",
                                                         CLUTTER_TYPE_ACTOR,
                                                         G_PARAM_READABLE));
+  g_object_class_install_property (gobject_class,
+                                   PROP_STAGE_INPUT_MODE,
+                                   g_param_spec_enum ("stage-input-mode",
+                                                      "Stage input mode",
+                                                      "The stage input mode",
+                                                      SHELL_TYPE_STAGE_INPUT_MODE,
+                                                      SHELL_STAGE_INPUT_MODE_NORMAL,
+                                                      G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
                                    PROP_WINDOW_GROUP,
                                    g_param_spec_object ("window-group",
@@ -341,6 +364,32 @@ shell_global_get (void)
   return the_object;
 }
 
+static void
+focus_window_changed (MetaDisplay *display,
+                      GParamSpec  *param,
+                      gpointer     user_data)
+{
+  ShellGlobal *global = user_data;
+
+  if (global->input_mode == SHELL_STAGE_INPUT_MODE_FOCUSED &&
+      meta_display_get_focus_window (display) != NULL)
+    shell_global_set_stage_input_mode (global, SHELL_STAGE_INPUT_MODE_NORMAL);
+}
+
+static void
+shell_global_focus_stage (ShellGlobal *global)
+{
+  Display *xdpy;
+  ClutterActor *stage;
+  Window xstage;
+
+  stage = mutter_plugin_get_stage (global->plugin);
+  xstage = clutter_x11_get_stage_window (CLUTTER_STAGE (stage));
+  xdpy = mutter_plugin_get_xdisplay (global->plugin);
+  XSetInputFocus (xdpy, xstage, RevertToPointerRoot,
+                  shell_global_get_current_time (global));
+}
+
 /**
  * shell_global_set_stage_input_mode:
  * @global: the #ShellGlobal
@@ -355,6 +404,12 @@ shell_global_get (void)
  * outside that region. When it is %SHELL_STAGE_INPUT_MODE_FULLSCREEN,
  * the stage absorbs all input.
  *
+ * When the input mode is %SHELL_STAGE_INPUT_MODE_FOCUSED, the pointer
+ * is handled as with %SHELL_STAGE_INPUT_MODE_NORMAL, but additionally
+ * the stage window has the keyboard focus. If the stage loses the
+ * focus (eg, because the user clicked into a window) the input mode
+ * will revert to %SHELL_STAGE_INPUT_MODE_NORMAL.
+ *
  * Note that whenever a mutter-internal Gtk widget has a pointer grab,
  * the shell behaves as though it was in
  * %SHELL_STAGE_INPUT_MODE_NONREACTIVE, to ensure that the widget gets
@@ -373,7 +428,96 @@ shell_global_set_stage_input_mode (ShellGlobal         *global,
   else
     mutter_plugin_set_stage_input_region (global->plugin, global->input_region);
 
-  global->input_mode = mode;
+  if (mode == SHELL_STAGE_INPUT_MODE_FOCUSED)
+    shell_global_focus_stage (global);
+
+  if (mode != global->input_mode)
+    {
+      global->input_mode = mode;
+      g_object_notify (G_OBJECT (global), "stage-input-mode");
+    }
+}
+
+/**
+ * shell_global_set_cursor:
+ * @global: A #ShellGlobal
+ * @type: the type of the cursor
+ *
+ * Set the cursor on the stage window.
+ */
+void
+shell_global_set_cursor (ShellGlobal *global,
+                         ShellCursor type)
+{
+  const char *name;
+  GdkCursor *cursor;
+
+  switch (type)
+    {
+    case SHELL_CURSOR_DND_IN_DRAG:
+      name = "dnd-none";
+      break;
+    case SHELL_CURSOR_DND_MOVE:
+      name = "dnd-move";
+      break;
+    case SHELL_CURSOR_DND_COPY:
+      name = "dnd-copy";
+      break;
+    case SHELL_CURSOR_DND_UNSUPPORTED_TARGET:
+      name = "dnd-none";
+      break;
+    default:
+      g_return_if_reached ();
+    }
+
+  cursor = gdk_cursor_new_from_name (gdk_display_get_default (), name);
+  if (!cursor)
+    {
+      GdkCursorType cursor_type;
+      switch (type)
+        {
+        case SHELL_CURSOR_DND_IN_DRAG:
+          cursor_type = GDK_FLEUR;
+          break;
+        case SHELL_CURSOR_DND_MOVE:
+          cursor_type = GDK_TARGET;
+          break;
+        case SHELL_CURSOR_DND_COPY:
+          cursor_type = GDK_PLUS;
+          break;
+        case SHELL_CURSOR_DND_UNSUPPORTED_TARGET:
+          cursor_type = GDK_X_CURSOR;
+          break;
+        default:
+          g_return_if_reached ();
+        }
+      cursor = gdk_cursor_new (cursor_type);
+    }
+  if (!global->stage_window)
+    {
+      ClutterStage *stage = CLUTTER_STAGE (mutter_plugin_get_stage (global->plugin));
+
+      global->stage_window = gdk_window_foreign_new (clutter_x11_get_stage_window (stage));
+    }
+
+  gdk_window_set_cursor (global->stage_window, cursor);
+
+  gdk_cursor_unref (cursor);
+}
+
+/**
+ * shell_global_unset_cursor:
+ * @global: A #ShellGlobal
+ *
+ * Unset the cursor on the stage window.
+ */
+void
+shell_global_unset_cursor (ShellGlobal  *global)
+{
+  if (!global->stage_window) /* cursor has never been set */
+    return;
+
+  gdk_window_set_cursor (global->stage_window, NULL);
 }
 
 /**
@@ -452,7 +596,7 @@ shell_global_get_gdk_screen (ShellGlobal *global)
  *
  * Gets the list of MutterWindows for the plugin's screen
  *
- * Return value: (element-type MutterWindow) (transfer none): the list of windows
+ * Return value: (element-type Meta.MutterWindow) (transfer none): the list of windows
  */
 GList *
 shell_global_get_windows (ShellGlobal *global)
@@ -535,6 +679,8 @@ _shell_global_set_plugin (ShellGlobal  *global,
                           MutterPlugin *plugin)
 {
   ClutterActor *stage;
+  MetaScreen *screen;
+  MetaDisplay *display;
 
   g_return_if_fail (SHELL_IS_GLOBAL (global));
   g_return_if_fail (global->plugin == NULL);
@@ -563,6 +709,11 @@ _shell_global_set_plugin (ShellGlobal  *global,
                                "clutter.stagePaintDone",
                                "End of stage page repaint",
                                "");
+
+  screen = mutter_plugin_get_screen (global->plugin);
+  display = meta_screen_get_display (screen);
+  g_signal_connect (display, "notify::focus-window",
+                    G_CALLBACK (focus_window_changed), global);
 }
 
 void
@@ -660,6 +811,8 @@ shell_global_add_extension_importer (ShellGlobal *global,
   JSContext *context = gjs_context_get_native_context (global->js_context);
   char *search_path[2] = { 0, 0 };
 
+  JS_BeginRequest (context);
+
   // This is a bit of a hack; ideally we'd be able to pass our target
   // object directly into this function, but introspection doesn't
   // support that at the moment.  Instead evaluate a string to get it.
@@ -679,18 +832,22 @@ shell_global_add_extension_importer (ShellGlobal *global,
                   G_IO_ERROR_FAILED,
                   "%s", message ? message : "(unknown)");
       g_free(message);
-      return FALSE;
+      goto out_error;
     }
 
   if (!JSVAL_IS_OBJECT (target_object))
     {
       g_error ("shell_global_add_extension_importer: invalid target object");
-      return FALSE;
+      goto out_error;
     }
 
   search_path[0] = (char*)directory;
   importer = gjs_define_importer (context, JSVAL_TO_OBJECT (target_object), target_property, (const char **)search_path, FALSE);
+  JS_EndRequest (context);
   return TRUE;
+ out_error:
+  JS_EndRequest (context);
+  return FALSE;
 }
 
 /* Code to close all file descriptors before we exec; copied from gspawn.c in GLib.
@@ -1303,7 +1460,7 @@ shell_get_event_state (ClutterEvent *event)
  *
  * Get the global GSettings instance.
  *
- * Return value: (transfer none) The GSettings object (transfer none).
+ * Return value: (transfer none): The GSettings object
  */
 GSettings *
 shell_global_get_settings (ShellGlobal *global)
@@ -1401,13 +1558,13 @@ shell_global_get_current_time (ShellGlobal *global)
 }
 
 /**
- * shell_global_get_app_launch_context:
+ * shell_global_create_app_launch_context:
  * @global: A #ShellGlobal
  *
  * Create a #GAppLaunchContext set up with the correct timestamp, and
  * targeted to activate on the current workspace.
  *
- * Return value: A new #GAppLaunchContext
+ * Return value: (transfer full): A new #GAppLaunchContext
  */
 GAppLaunchContext *
 shell_global_create_app_launch_context (ShellGlobal *global)
@@ -1454,7 +1611,8 @@ shell_global_set_property_mutable (ShellGlobal *global,
   jsuint attrs;
   JSBool found;
 
-  JS_AddRoot (context, &val);
+  JS_BeginRequest (context);
+  JS_AddValueRoot (context, &val);
 
   parts = g_strsplit (object, ".", -1);
   obj = JS_GetGlobalObject (context);
@@ -1463,30 +1621,31 @@ shell_global_set_property_mutable (ShellGlobal *global,
       if (!JS_GetProperty (context, obj, parts[i], &val))
         {
           g_strfreev (parts);
-          JS_RemoveRoot (context, &val);
-          gjs_log_exception (context, NULL);
-          return FALSE;
+          goto out_error;
         }
       obj = JSVAL_TO_OBJECT (val);
     }
   g_strfreev (parts);
 
   if (!JS_GetPropertyAttributes (context, obj, property, &attrs, &found) || !found)
-    {
-      JS_RemoveRoot (context, &val);
-      gjs_log_exception (context, NULL);
-      return FALSE;
-    }
+    goto out_error;
 
   if (mutable)
     attrs &= ~(JSPROP_PERMANENT | JSPROP_READONLY);
   else
     attrs |= (JSPROP_PERMANENT | JSPROP_READONLY);
 
-  JS_SetPropertyAttributes (context, obj, property, attrs, &found);
+  if (!JS_SetPropertyAttributes (context, obj, property, attrs, &found))
+    goto out_error;
 
-  JS_RemoveRoot (context, &val);
-  return !gjs_log_exception (context, NULL);
+  JS_RemoveValueRoot (context, &val);
+  JS_EndRequest (context);
+  return TRUE;
+ out_error:
+  gjs_log_exception (context, NULL);
+  JS_RemoveValueRoot (context, &val);
+  JS_EndRequest (context);
+  return FALSE;
 }
 
 typedef struct
