@@ -32,10 +32,20 @@ GType            shell_global_get_type            (void) G_GNUC_CONST;
 gboolean shell_clutter_texture_set_from_pixbuf (ClutterTexture *texture,
                                                 GdkPixbuf      *pixbuf);
 
-ClutterActor *shell_get_event_related(ClutterEvent *event);
-
 ShellGlobal *shell_global_get (void);
 
+typedef enum {
+  SHELL_CURSOR_DND_IN_DRAG,
+  SHELL_CURSOR_DND_UNSUPPORTED_TARGET,
+  SHELL_CURSOR_DND_MOVE,
+  SHELL_CURSOR_DND_COPY
+} ShellCursor;
+
+void shell_global_set_cursor (ShellGlobal *global,
+                              ShellCursor type);
+
+void shell_global_unset_cursor (ShellGlobal  *global);
+
 MetaScreen *shell_global_get_screen (ShellGlobal  *global);
 
 GdkScreen *shell_global_get_gdk_screen (ShellGlobal  *global);
@@ -51,6 +61,7 @@ void shell_global_grab_dbus_service (ShellGlobal *global);
 typedef enum {
   SHELL_STAGE_INPUT_MODE_NONREACTIVE,
   SHELL_STAGE_INPUT_MODE_NORMAL,
+  SHELL_STAGE_INPUT_MODE_FOCUSED,
   SHELL_STAGE_INPUT_MODE_FULLSCREEN
 } ShellStageInputMode;
 
@@ -76,7 +87,7 @@ void shell_global_gc (ShellGlobal *global);
 
 void shell_global_maybe_gc (ShellGlobal *global);
 
-void shell_global_format_time_relative_pretty (ShellGlobal *global, guint delta, char **text, guint *update_time);
+void shell_global_format_time_relative_pretty (ShellGlobal *global, guint delta, char **text, guint *next_update);
 
 ClutterActor *shell_global_create_root_pixmap_actor (ShellGlobal *global);
 
