@@ -29,200 +29,15 @@
 */
 
 #include <stdlib.h>
-#include <string.h>
 #include <math.h>
 
+#include "st-shadow.h"
+#include "st-private.h"
 #include "st-theme-private.h"
 #include "st-theme-context.h"
 #include "st-texture-cache.h"
 #include "st-theme-node-private.h"
 
-/*****
- * Shadows
- *****/
-
-static gdouble *
-calculate_gaussian_kernel (gdouble   sigma,
-                           guint     n_values)
-{
-  gdouble *ret, sum;
-  gdouble exp_divisor;
-  gint half, i;
-
-  g_return_val_if_fail ((int) sigma > 0, NULL);
-
-  half = n_values / 2;
-
-  ret = g_malloc (n_values * sizeof (gdouble));
-  sum = 0.0;
-
-  exp_divisor = 2 * sigma * sigma;
-
-  /* n_values of 1D Gauss function */
-  for (i = 0; i < n_values; i++)
-    {
-      ret[i] = exp (-(i - half) * (i - half) / exp_divisor);
-      sum += ret[i];
-    }
-
-  /* normalize */
-  for (i = 0; i < n_values; i++)
-    ret[i] /= sum;
-
-  return ret;
-}
-
-static CoglHandle
-create_shadow_material (StThemeNode  *node,
-                        CoglHandle    src_texture)
-{
-  CoglColor   color;
-  CoglHandle  material;
-  CoglHandle  texture;
-  StShadow   *shadow_spec;
-  guchar     *pixels_in, *pixels_out;
-  gint        width_in, height_in, rowstride_in;
-  gint        width_out, height_out, rowstride_out;
-  float       sigma;
-
-  shadow_spec = st_theme_node_get_shadow (node);
-  if (!shadow_spec)
-    return COGL_INVALID_HANDLE;
-
-  /* we use an approximation of the sigma - blur radius relationship used
-     in Firefox for doing SVG blurs; see
-     http://mxr.mozilla.org/mozilla-central/source/gfx/thebes/src/gfxBlur.cpp#280
-  */
-  sigma = shadow_spec->blur / 1.9;
-
-  width_in  = cogl_texture_get_width  (src_texture);
-  height_in = cogl_texture_get_height (src_texture);
-  rowstride_in = (width_in + 3) & ~3;
-
-  pixels_in  = g_malloc0 (rowstride_in * height_in);
-
-  cogl_texture_get_data (src_texture, COGL_PIXEL_FORMAT_A_8,
-                         rowstride_in, pixels_in);
-
-  if ((guint) shadow_spec->blur == 0)
-    {
-      width_out  = width_in;
-      height_out = height_in;
-      rowstride_out = rowstride_in;
-      pixels_out = g_memdup (pixels_in, rowstride_out * height_out);
-    }
-  else
-    {
-      gdouble *kernel;
-      guchar  *line;
-      gint     n_values, half;
-      gint     x_in, y_in, x_out, y_out, i;
-
-      n_values = (gint) 5 * sigma;
-      half = n_values / 2;
-
-      width_out  = width_in  + 2 * half;
-      height_out = height_in + 2 * half;
-      rowstride_out = (width_out + 3) & ~3;
-
-      pixels_out = g_malloc0 (rowstride_out * height_out);
-      line       = g_malloc0 (rowstride_out);
-
-      kernel = calculate_gaussian_kernel (sigma, n_values);
-
-      /* vertical blur */
-      for (x_in = 0; x_in < width_in; x_in++)
-        for (y_out = 0; y_out < height_out; y_out++)
-          {
-            guchar *pixel_in, *pixel_out;
-            gint i0, i1;
-
-            y_in = y_out - half;
-
-            /* We read from the source at 'y = y_in + i - half'; clamp the
-             * full i range [0, n_values) so that y is in [0, height_in).
-             */
-            i0 = MAX (half - y_in, 0);
-            i1 = MIN (height_in + half - y_in, n_values);
-
-            pixel_in  =  pixels_in + (y_in + i0 - half) * rowstride_in + x_in;
-            pixel_out =  pixels_out + y_out * rowstride_out + (x_in + half);
-
-            for (i = i0; i < i1; i++)
-              {
-                *pixel_out += *pixel_in * kernel[i];
-                pixel_in += rowstride_in;
-              }
-          }
-
-      /* horizontal blur */
-      for (y_out = 0; y_out < height_out; y_out++)
-        {
-          memcpy (line, pixels_out + y_out * rowstride_out, rowstride_out);
-
-          for (x_out = 0; x_out < width_out; x_out++)
-            {
-              gint i0, i1;
-              guchar *pixel_out, *pixel_in;
-
-              /* We read from the source at 'x = x_out + i - half'; clamp the
-               * full i range [0, n_values) so that x is in [0, width_out).
-               */
-              i0 = MAX (half - x_out, 0);
-              i1 = MIN (width_out + half - x_out, n_values);
-
-              pixel_in  = line + x_out + i0 - half;
-              pixel_out = pixels_out + rowstride_out * y_out + x_out;
-
-              *pixel_out = 0;
-              for (i = i0; i < i1; i++)
-                {
-                  *pixel_out += *pixel_in * kernel[i];
-                  pixel_in++;
-                }
-            }
-        }
-      g_free (kernel);
-      g_free (line);
-    }
-
-  texture = cogl_texture_new_from_data (width_out,
-                                        height_out,
-                                        COGL_TEXTURE_NONE,
-                                        COGL_PIXEL_FORMAT_A_8,
-                                        COGL_PIXEL_FORMAT_A_8,
-                                        rowstride_out,
-                                        pixels_out);
-
-  g_free (pixels_in);
-  g_free (pixels_out);
-
-  material = cogl_material_new ();
-
-  cogl_color_set_from_4ub (&color,
-                           shadow_spec->color.red, shadow_spec->color.green,
-                           shadow_spec->color.blue, shadow_spec->color.alpha);
-  cogl_color_premultiply (&color);
-
-  cogl_material_set_layer_combine_constant (material, 0, &color);
-  cogl_material_set_layer (material, 0, texture);
-
-  /* We ignore the material color, which encodes the overall opacity of the
-   * actor, so setting an ancestor of the shadow to partially opaque won't
-   * work. The easiest way to fix this would be to not set the combine or constant color
-   * here, then in paint set the material color to the shadow_spec_color *
-   * paint_opacity.*/
-  cogl_material_set_layer_combine (material, 0,
-                                   "RGBA = MODULATE (CONSTANT, TEXTURE[A])",
-                                   NULL);
-
-
-  cogl_handle_unref (texture);
-
-  return material;
-}
-
-
 /****
  * Rounded corners
  ****/
@@ -474,8 +289,8 @@ get_background_position (StThemeNode             *self,
   w = cogl_texture_get_width (self->background_texture);
   h = cogl_texture_get_height (self->background_texture);
 
-  /* scale the background into the allocated bounds */
-  if (w > result->x2 || h > result->y2)
+  /* scale the background into the allocated bounds, when not being absolutely positioned */
+  if ((w > result->x2 || h > result->y2) && !self->background_position_set)
     {
       gint new_h, new_w, offset;
       gint box_w, box_h;
@@ -508,9 +323,18 @@ get_background_position (StThemeNode             *self,
     }
   else
     {
-      /* center the background on the widget */
-      result->x1 = (int)(((allocation->x2 - allocation->x1) / 2) - (w / 2));
-      result->y1 = (int)(((allocation->y2 - allocation->y1) / 2) - (h / 2));
+      /* honor the specified position if any */
+      if (self->background_position_set)
+        {
+          result->x1 = self->background_position_x;
+          result->y1 = self->background_position_y;
+        }
+      else
+        {
+          /* center the background on the widget */
+          result->x1 = (int)(((allocation->x2 - allocation->x1) / 2) - (w / 2));
+          result->y1 = (int)(((allocation->y2 - allocation->y1) / 2) - (h / 2));
+        }
       result->x2 = result->x1 + w;
       result->y2 = result->y1 + h;
     }
@@ -749,7 +573,8 @@ st_theme_node_render_resources (StThemeNode   *node,
   if (shadow_spec)
     {
       if (node->border_texture != COGL_INVALID_HANDLE)
-        node->border_shadow_material = create_shadow_material (node, node->border_texture);
+        node->border_shadow_material = _st_create_shadow_material (shadow_spec,
+                                                                   node->border_texture);
       else if (node->background_color.alpha > 0 ||
                node->border_width[ST_SIDE_TOP] > 0 ||
                node->border_width[ST_SIDE_LEFT] > 0 ||
@@ -774,8 +599,8 @@ st_theme_node_render_resources (StThemeNode   *node,
               cogl_pop_framebuffer ();
               cogl_handle_unref (offscreen);
 
-              node->border_shadow_material = create_shadow_material (node,
-                                                                     buffer);
+              node->border_shadow_material = _st_create_shadow_material (shadow_spec,
+                                                                         buffer);
             }
           cogl_handle_unref (buffer);
         }
@@ -789,7 +614,8 @@ st_theme_node_render_resources (StThemeNode   *node,
 
       if (shadow_spec)
         {
-          node->background_shadow_material = create_shadow_material (node, node->background_texture);
+          node->background_shadow_material = _st_create_shadow_material (shadow_spec,
+                                                                         node->background_texture);
         }
     }
 
@@ -1036,6 +862,54 @@ st_theme_node_paint_sliced_border_image (StThemeNode           *node,
   cogl_handle_unref (material);
 }
 
+static void
+st_theme_node_paint_outline (StThemeNode           *node,
+                             const ClutterActorBox *box,
+                             guint8                 paint_opacity)
+
+{
+  float width, height;
+  int outline_width;
+  ClutterColor outline_color, effective_outline;
+
+  width = box->x2 - box->x1;
+  height = box->y2 - box->y1;
+
+  outline_width = st_theme_node_get_outline_width (node);
+  if (outline_width == 0)
+    return;
+
+  st_theme_node_get_outline_color (node, &outline_color);
+  over (&outline_color, &node->background_color, &effective_outline);
+
+  cogl_set_source_color4ub (effective_outline.red,
+                            effective_outline.green,
+                            effective_outline.blue,
+                            paint_opacity * effective_outline.alpha / 255);
+
+  /* The outline is drawn just outside the border, which means just
+   * outside the allocation box. This means that in some situations
+   * involving clip_to_allocation or the screen edges, you won't be
+   * able to see the outline. In practice, it works well enough.
+   */
+
+  /* NORTH */
+  cogl_rectangle (-outline_width, -outline_width,
+                  width + outline_width, 0);
+
+  /* EAST */
+  cogl_rectangle (width, 0,
+                  width + outline_width, height);
+
+  /* SOUTH */
+  cogl_rectangle (-outline_width, height,
+                  width + outline_width, height + outline_width);
+
+  /* WEST */
+  cogl_rectangle (-outline_width, 0,
+                  0, height);
+}
+
 void
 st_theme_node_paint (StThemeNode           *node,
                      const ClutterActorBox *box,
@@ -1087,28 +961,10 @@ st_theme_node_paint (StThemeNode           *node,
    */
 
   if (node->border_shadow_material)
-    {
-      StShadow *shadow_spec;
-      ClutterActorBox shadow_box;
-
-      shadow_spec = node->shadow;
-
-      shadow_box.x1 = allocation.x1 + shadow_spec->xoffset
-                      - shadow_spec->blur - shadow_spec->spread;
-      shadow_box.y1 = allocation.y1 + shadow_spec->yoffset
-                      - shadow_spec->blur - shadow_spec->spread;
-      shadow_box.x2 = allocation.x2 + shadow_spec->xoffset
-                      + shadow_spec->blur + shadow_spec->spread;
-      shadow_box.y2 = allocation.y2 + shadow_spec->yoffset
-                      + shadow_spec->blur + shadow_spec->spread;
-
-      cogl_material_set_color4ub (node->border_shadow_material,
-                                  paint_opacity, paint_opacity, paint_opacity, paint_opacity);
-
-      cogl_set_source (node->border_shadow_material);
-      cogl_rectangle_with_texture_coords (shadow_box.x1, shadow_box.y1, shadow_box.x2, shadow_box.y2,
-                                          0, 0, 1, 1);
-    }
+    _st_paint_shadow_with_opacity (node->shadow,
+                                   node->border_shadow_material,
+                                   &allocation,
+                                   paint_opacity);
 
   if (node->border_texture != COGL_INVALID_HANDLE)
     {
@@ -1121,6 +977,8 @@ st_theme_node_paint (StThemeNode           *node,
   else
     st_theme_node_paint_borders (node, box, paint_opacity);
 
+  st_theme_node_paint_outline (node, box, paint_opacity);
+
   if (node->background_texture != COGL_INVALID_HANDLE)
     {
       ClutterActorBox background_box;
@@ -1140,29 +998,48 @@ st_theme_node_paint (StThemeNode           *node,
        * like boder and background color into account).
        */
       if (node->background_shadow_material != COGL_INVALID_HANDLE)
-        {
-          StShadow *shadow_spec;
-          ClutterActorBox shadow_box;
-
-          shadow_spec = node->shadow;
-
-          shadow_box.x1 = background_box.x1 + shadow_spec->xoffset
-                          - shadow_spec->blur - shadow_spec->spread;
-          shadow_box.y1 = background_box.y1 + shadow_spec->yoffset
-                          - shadow_spec->blur - shadow_spec->spread;
-          shadow_box.x2 = background_box.x2 + shadow_spec->xoffset
-                          + shadow_spec->blur + shadow_spec->spread;
-          shadow_box.y2 = background_box.y2 + shadow_spec->yoffset
-                          + shadow_spec->blur + shadow_spec->spread;
-
-          cogl_material_set_color4ub (node->background_shadow_material,
-                                      paint_opacity, paint_opacity, paint_opacity, paint_opacity);
-
-          cogl_set_source (node->background_shadow_material);
-          cogl_rectangle_with_texture_coords (shadow_box.x1, shadow_box.y1, shadow_box.x2, shadow_box.y2,
-                                              0, 0, 1, 1);
-        }
+        _st_paint_shadow_with_opacity (node->shadow,
+                                       node->background_shadow_material,
+                                       &background_box,
+                                       paint_opacity);
 
       paint_texture_with_opacity (node->background_texture, &background_box, paint_opacity);
     }
 }
+
+/**
+ * st_theme_node_copy_cached_paint_state:
+ * @node: a #StThemeNode
+ * @other: a different #StThemeNode
+ *
+ * Copy cached painting state from @other to @node. This function can be used to
+ * optimize redrawing cached background images when the style on an element changess
+ * in a way that doesn't affect background drawing. This function must only be called
+ * if st_theme_node_paint_equal (node, other) returns %TRUE.
+ */
+void
+st_theme_node_copy_cached_paint_state (StThemeNode *node,
+                                       StThemeNode *other)
+{
+  g_return_if_fail (ST_IS_THEME_NODE (node));
+  g_return_if_fail (ST_IS_THEME_NODE (other));
+
+  /* Check omitted for speed: */
+  /* g_return_if_fail (st_theme_node_paint_equal (node, other)); */
+
+  _st_theme_node_free_drawing_state (node);
+
+  node->alloc_width = other->alloc_width;
+  node->alloc_height = other->alloc_height;
+
+  if (other->background_shadow_material)
+    node->background_shadow_material = cogl_handle_ref (other->background_shadow_material);
+  if (other->border_shadow_material)
+    node->border_shadow_material = cogl_handle_ref (other->border_shadow_material);
+  if (other->background_texture)
+    node->background_texture = cogl_handle_ref (other->background_texture);
+  if (other->border_texture)
+    node->border_texture = cogl_handle_ref (other->border_texture);
+  if (other->corner_texture)
+    node->corner_texture = cogl_handle_ref (other->corner_texture);
+}
