@@ -538,17 +538,18 @@ na_tray_manager_unmanage (NaTrayManager *manager)
 {
 #ifdef GDK_WINDOWING_X11
   GdkDisplay *display;
-  GdkWindow  *window;
   guint32     timestamp;
   GtkWidget  *invisible;
+  GdkWindow  *window;
 
   if (manager->invisible == NULL)
     return;
 
   invisible = manager->invisible;
   window = gtk_widget_get_window (invisible);
+
   g_assert (GTK_IS_INVISIBLE (invisible));
-  g_assert (gtk_widget_get_realized (GTK_WIDGET (invisible)));
+  g_assert (gtk_widget_get_realized (invisible));
   g_assert (GDK_IS_WINDOW (window));
   
   display = gtk_widget_get_display (invisible);
@@ -580,16 +581,14 @@ static void
 na_tray_manager_set_orientation_property (NaTrayManager *manager)
 {
 #ifdef GDK_WINDOWING_X11
-  GdkDisplay *display;
   GdkWindow  *window;
+  GdkDisplay *display;
   Atom        orientation_atom;
   gulong      data[1];
 
   if (!manager->invisible)
     return;
-
-  window = gtk_widget_get_window (GTK_WIDGET (manager->invisible));
-
+  window = gtk_widget_get_window (manager->invisible);
   if (!window)
     return;
 
@@ -614,17 +613,15 @@ static void
 na_tray_manager_set_visual_property (NaTrayManager *manager)
 {
 #ifdef GDK_WINDOWING_X11
-  GdkDisplay *display;
   GdkWindow  *window;
+  GdkDisplay *display;
   Visual     *xvisual;
   Atom        visual_atom;
   gulong      data[1];
 
   if (!manager->invisible)
     return;
-
-  window = gtk_widget_get_window (GTK_WIDGET (manager->invisible));
-
+  window = gtk_widget_get_window (manager->invisible);
   if (!window)
     return;
 
@@ -642,20 +639,9 @@ na_tray_manager_set_visual_property (NaTrayManager *manager)
 
   if (gdk_screen_get_rgba_visual (manager->screen) != NULL &&
       gdk_display_supports_composite (display))
-    {
-      xvisual = GDK_VISUAL_XVISUAL (gdk_screen_get_rgba_visual (manager->screen));
-    }
+    xvisual = GDK_VISUAL_XVISUAL (gdk_screen_get_rgba_visual (manager->screen));
   else
-    {
-      /* We actually want the visual of the tray where the icons will
-       * be embedded. In almost all cases, this will be the same as the visual
-       * of the screen.
-       */
-      GdkColormap *colormap;
-
-      colormap = gdk_screen_get_default_colormap (manager->screen);
-      xvisual = GDK_VISUAL_XVISUAL (gdk_colormap_get_visual (colormap));
-    }
+    xvisual = GDK_VISUAL_XVISUAL (gdk_screen_get_system_visual (manager->screen));
 
   data[0] = XVisualIDFromVisual (xvisual);
 
@@ -675,9 +661,9 @@ na_tray_manager_manage_screen_x11 (NaTrayManager *manager,
 				   GdkScreen     *screen)
 {
   GdkDisplay *display;
-  GdkWindow  *window;
   Screen     *xscreen;
   GtkWidget  *invisible;
+  GdkWindow  *window;
   char       *selection_atom_name;
   guint32     timestamp;
   
@@ -699,8 +685,6 @@ na_tray_manager_manage_screen_x11 (NaTrayManager *manager,
   
   invisible = gtk_invisible_new_for_screen (screen);
   gtk_widget_realize (invisible);
-
-  window = gtk_widget_get_window (GTK_WIDGET (invisible));
   
   gtk_widget_add_events (invisible,
                          GDK_PROPERTY_CHANGE_MASK | GDK_STRUCTURE_MASK);
@@ -716,6 +700,8 @@ na_tray_manager_manage_screen_x11 (NaTrayManager *manager,
   na_tray_manager_set_orientation_property (manager);
   na_tray_manager_set_visual_property (manager);
   
+  window = gtk_widget_get_window (invisible);
+
   timestamp = gdk_x11_get_server_time (window);
 
   /* Check if we could set the selection owner successfully */
